/* 
   txt2snd - tab-separated-value to soundfile converter

   Copyright (C) 2007 Robin Gareus

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 
   as published by the Free Software Foundation;

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <math.h>
#include <sndfile.h>

#define VERSION "0.1"

static void usage (int status);
char *program_name;

static struct option const long_options[] =
{
  {"samplerate", required_argument, 0, 's'},
  {"Vmult", required_argument, 0, 'm'},
  {"Voffset", required_argument, 0, 'o'},
  {"verbose", no_argument, 0, 'v'},
  {"debuglevel", required_argument, 0, 'd'},
 
};

int o_samplerate = 48000;
float o_mult = 1.0;
float o_off = 0.0;
int o_verbose = 0;
int o_debug = 0;


static int decode_switches (int argc, char **argv) {
  int c;

  while ((c = getopt_long (argc, argv, 
			   "o:"	/* offset */
			   "m:"	/* offset */
			   "s:"	/* samplerate */
			   "v"	/* verbose */
			   "d:"	/* debug-level */
			   "h"	/* help */
			   "V",	/* version */
			   long_options, (int *) 0)) != EOF)
    {
      switch (c)
	{
	case 'm':
	  o_mult = atof(optarg);
	  break;
	case 'o':
	  o_off = atof(optarg);
	  break;
	case 's':
	  o_samplerate = atoi(optarg);
	  break;
	case 'd':
	  o_debug = atoi(optarg);
	  break;
	case 'v':
	  o_verbose = 1;
	  break;
	case 'V':
	  printf ("%s %s\n", program_name, VERSION);
	  exit (0);
	case 'h':
	  usage (0);
	default:
	  usage (EXIT_FAILURE);
	}
    }

  return optind;
}

int main (int argc, char **argv) {
  program_name = argv[0];

  int i;
  i = decode_switches (argc, argv);
  if (i != argc-1) usage(1);

  SNDFILE * outfile ;
  SF_INFO sfinfo ;

  sfinfo.samplerate = o_samplerate;
  sfinfo.channels = 1;
  sfinfo.frames = 0;
  sfinfo.format = (SF_FORMAT_WAV | SF_FORMAT_PCM_24) ;

  if ((outfile = sf_open (argv[i], SFM_WRITE, &sfinfo)) == NULL) {
    printf ("Error: Not able to open output file '%s'\n", argv[i]);
    exit (1) ;
  }

#if 1
  sf_command (outfile, SFC_SET_UPDATE_HEADER_AUTO, NULL, SF_TRUE) ;
  sf_command (outfile, SFC_SET_CLIPPING, NULL, SF_TRUE) ;
#endif

  float time,in,out;
  int  idx, gndcnt;
  int duptot, dupcnt;
  int mistot;
  float min,max,gnd,dupbuf;
  int laststamp = -1;
  int timestamp = 0;
  min=max=0.0;
  gnd=0.0;
  gndcnt=0;
  mistot=duptot=dupcnt=0;

  do {  
  /* format (tab-seprarated)
   * idx     time[ms]        [input/ignored] [output]
   * 11      8.916652e-05    1.846313e-02    -7.477363e-01
   */
    if (4!=(fscanf(stdin,"%i\t%f\t%f\t%f\n", &idx,&time,&in,&out)))break;
    if (o_debug&8)    
      printf("read idx:%i t=%f i=%f o=%f\n",idx,time*48000.0, in, out);

    timestamp= (int) rint(time * (double)o_samplerate);

    float val = o_off + out * o_mult;

    if (val< min) min=val;
    if (val> max) max=val;

    if (timestamp > laststamp) {
      int i;

      if (dupcnt == 1 && (timestamp-laststamp) == 2) {
	sf_writef_float (outfile, &dupbuf, 1) ;
	dupcnt--;
	gnd+=dupbuf; gndcnt++; laststamp++;
	if (o_verbose) 
	  printf("---- worked around time rounding.\n");
	if (o_debug&2 ) 
	  printf("buf: %7i | %7i - 1 frame(s) - [%f]\n",idx-1,laststamp, dupbuf);
      }

      if (dupcnt > 0) { 
        duptot++;
	if (o_verbose) 
	  printf("!--- ignored %i dup. for sample %i\n",dupcnt, laststamp);
      }

      if (o_debug&2 ) 
	printf("save %7i | %7i - %i frame(s) - [%f]\n",idx,timestamp, (timestamp-laststamp), val);
      if (o_verbose && (timestamp-laststamp)>1)
	printf("!!!! inserted %i frame(s) @%i [%f]\n",(timestamp-laststamp)-1,timestamp,val);

      for (i=0;i<(timestamp-laststamp);i++) {
	sf_writef_float (outfile, &val, 1) ;
	gnd+=val; gndcnt++;
      }

      mistot+=timestamp-laststamp-1;
      laststamp=timestamp;
      dupcnt=0;
    } else {
      dupcnt++;
      dupbuf=val;
      if ((o_debug&1)) 
	printf("dup! %7i | %7i [%f]\n",idx,timestamp, val);
    }
    
  } while(1);

  sf_close(outfile);

  printf(" min:%f - max:%f (2/delta: %f) gnd:%f\n",min,max, 2.0/(max-min), (gnd/(double)gndcnt));
  printf(" duplicate/missing sample info: %i/%i\n",duptot,mistot);
  return(0);
}


static void usage (int status) {
  printf ("%s %s - \
tab-seperated-text to sound file converter\n", program_name, VERSION);
  printf ("reads from <stdin> and writes 16bps signed-le wav files.\n");
  printf ("Usage: %s [OPTION]... <outfile>...\n", program_name);
  printf ("\
Options:\n\
  -s <int>                   samplerate [48000]\n\
  -o <float>                 GND offset  [0.0]\n\
  -m <float>                 amplitude multiplier [1.0]\n\
  -v, --verbose              print processing information\n\
  -d <int>, --debug <int>    debug level (bits0-3)\n\
  -h, --help                 display this help and exit\n\
  -V, --version              output version information and exit\n\
  \n\
Input Format:\n\
  4 tabulator separated values per line:\n\
  <idx/int> <time|ms/float> <ignored/float> <value/float>\n\
11      8.916652e-05    1.846313e-02    -7.477363e-01\n\
\n\
");
  exit (status);
}

/* vi:set ts=8 sts=2 sw=2: */

